@@ -5,7 +5,6 @@ use 5.006002;
 use strict;
 use warnings;
 
-
 use charnames qw{ :full };
 
 use Carp ();
@@ -18,21 +17,23 @@ use Text::Abbrev();
 use Exporter ();
 our @ISA = qw{ Exporter };
 
-our $VERSION = '0.000_009';
+our $VERSION = '0.000_013';
 
 our @EXPORT_OK = qw{
+    __am_or_pm
     __date_to_day_of_year
     __day_of_year_to_date
     __day_of_week
     __format
     __is_leap_year
-    __holiday_name __holiday_name_to_number __holiday_short
-    __month_name __month_name_to_number __month_short
+    __holiday_name __holiday_name_to_number __holiday_abbr
+    __month_name __month_name_to_number __month_abbr
     __on_date __on_date_accented
-    __quarter
+    __quarter __quarter_name __quarter_abbr
     __rata_die_to_year_day
-    __trad_weekday_name __trad_weekday_short
-    __weekday_name __weekday_short
+    __trad_weekday_name __trad_weekday_abbr
+    __valid_date_class
+    __weekday_name __weekday_abbr
     __week_of_year
     __year_day_to_rata_die
     GREGORIAN_RATA_DIE_TO_SHIRE
@@ -41,6 +42,7 @@ our %EXPORT_TAGS = (
     all	=> \@EXPORT_OK,
 );
 
+use constant ARRAY_REF	=> ref [];
 use constant CODE_REF	=> ref sub {};
 use constant HASH_REF	=> ref {};
 
@@ -52,13 +54,26 @@ use constant OVERLITHE_DAY_OF_YEAR	=> 184;
 
 use constant GREGORIAN_RATA_DIE_TO_SHIRE	=> 1995694;
 
+{
+    my @name = qw{ AM PM };
+
+    my $validate = _make_validator( qw{ UInt } );
+
+    sub __am_or_pm {
+	my ( $hour ) = $validate->( @_ );
+	return $name[ $hour < 12 ? 0 : 1 ];
+    }
+}
+
 {
 
     my @holiday = ( undef, 1, 7, 0, 0, 1, 7 );
     my @month_zero = ( undef, 0, 2, 4, 6, 1, 3, 0, 2, 4, 6, 1, 3 );
 
+    my $validate = _make_validator( qw{ UInt UInt } );
+
     sub __day_of_week {
-	my ( $month, $day ) = @_;
+	my ( $month, $day ) = $validate->( @_ );
 	$month
 	    or return $holiday[$day];
 	return ( $month_zero[$month] + $day ) % 7 + 1;
@@ -70,8 +85,10 @@ use constant GREGORIAN_RATA_DIE_TO_SHIRE	=> 1995694;
     my @month_zero = ( undef, 1, 31, 61, 91, 121, 151, 185, 215, 245,
 	275, 305, 335 );
 
+    my $validate_d2doy = _make_validator( qw{ UInt UInt UInt } );
+
     sub __date_to_day_of_year {
-	my ( $year, $month, $day ) = @_;
+	my ( $year, $month, $day ) = $validate_d2doy->( @_ );
 
 	my $yd = $month ? $month_zero[$month] + $day :
 	$holiday_day[$day];
@@ -86,8 +103,10 @@ use constant GREGORIAN_RATA_DIE_TO_SHIRE	=> 1995694;
 	return $yd;
     }
 
+    my $validate_doy2d = _make_validator( qw{ UInt UInt } );
+
     sub __day_of_year_to_date {
-	my ( $year, $yd ) = @_;
+	my ( $year, $yd ) = $validate_doy2d->( @_ );
 
 	unless ( __is_leap_year( $year ) ) {
 	    $yd >= OVERLITHE_DAY_OF_YEAR
@@ -112,33 +131,13 @@ use constant GREGORIAN_RATA_DIE_TO_SHIRE	=> 1995694;
 }
 
 {
-    use constant FORMAT_DATE_ERROR => 'Date must be object or hash';
+
+    my $validate = _make_validator( qw{ Hash|Object Scalar } );
 
     sub __format {
-	my ( $date, $tplt ) = @_;
+	my ( $date, $tplt ) = $validate->( @_ );
 
-	my $ref = ref $date
-	    or Carp::croak( FORMAT_DATE_ERROR );
-
-	if ( HASH_REF eq $ref ) {
-	    my %hash = %{ $date };
-	    if ( $hash{month} ) {
-		$hash{holiday} = 0;
-	    } else {
-		$hash{holiday}
-		    or $hash{holiday} = $hash{day};
-		$hash{day} = 0;
-	    }
-	    $date = bless \%hash, join '::', __PACKAGE__, 'Date';
-	}
-
-	{
-	    local $@ = undef;
-	    eval {
-		$date->can( 'isa' );
-		1;
-	    } or Carp::croak( FORMAT_DATE_ERROR );
-	}
+	$date = _make_date_object( $date );
 
 	my $ctx = {
 	    prefix_new_line_unless_empty	=> 0,
@@ -168,7 +167,7 @@ sub _fmt_cond {
 
     my $inx = 0;
     defined $cond[1]
-	and not $date->month()
+	and not $date->month_number()
 	and $inx = 1;
     defined $cond[2]
 	and not __day_of_week( _fmt_get_md( $date ) )
@@ -179,40 +178,41 @@ sub _fmt_cond {
 
 sub _fmt_get_md {
     my ( $date ) = @_;
-    my $month = $date->month() || 0;
-    my $day = $month ? $date->day() : $date->holiday();
+    my $month = $date->month_number() || 0;
+    my $day = $month ? $date->day_number() : $date->holiday_number();
     return ( $month, $day );
 }
 
 {
     my %spec = (
-	A	=> sub { __weekday_name( $_[0]->day_of_week() ) },
-	a	=> sub { __weekday_short( $_[0]->day_of_week() ) },
-	B	=> sub { __month_name( $_[0]->month() ) },
-	b	=> sub { __month_short( $_[0]->month() ) },
+	A	=> sub { $_[0]->traditional() ?
+		    __trad_weekday_name( $_[0]->weekday_number() ) :
+		    __weekday_name( $_[0]->weekday_number() );
+		},
+	a	=> sub { $_[0]->traditional() ?
+		    __trad_weekday_abbr( $_[0]->weekday_number() ) :
+		    __weekday_abbr( $_[0]->weekday_number() );
+		},
+	B	=> sub { __month_name( $_[0]->month_number() ) },
+	b	=> sub { __month_abbr( $_[0]->month_number() ) },
 	C	=> sub {
-		    $_[2] = int( $_[0]->year() / 100 );
+		    $_[2] = int( $_[0]->year_number() / 100 );
 		    goto &_fmt_number_02;
 		},
 	c	=> sub { __format( $_[0], '%{{%a %x||||%x}} %X' ) },
 	D	=> sub { __format( $_[0], '%{{%m/%d||%Ee}}/%y' ) },
 	d	=> sub {
-		    $_[2] = $_[0]->day() || $_[0]->holiday();
+		    $_[2] = $_[0]->day_number() || $_[0]->holiday_number();
 		    goto &_fmt_number_02;
 		},
-	EA	=> sub { __trad_weekday_name( $_[0]->day_of_week() ) },
-	Ea	=> sub { __trad_weekday_short( $_[0]->day_of_week() ) },
-	ED	=> sub { $_[1]{on_date_accented} = 1;
-		    goto &_fmt_on_date;
-		},
 	Ed	=> \&_fmt_on_date,
-	EE	=> sub { __holiday_name( $_[0]->holiday() || 0 ) },
-	Ee	=> sub { __holiday_short( $_[0]->holiday() || 0 ) },
+	EE	=> sub { __holiday_name( $_[0]->holiday_number() ) },
+	Ee	=> sub { __holiday_abbr( $_[0]->holiday_number() ) },
 	En	=> sub { $_[1]{prefix_new_line_unless_empty}++; '' },
 	Ex	=> sub { __format( $_[0],
-		'%{{%A %-e %B %Y||%A %EE %Y||%EE %Y}}' ) },
+		    '%{{%A %-e %B %Y||%A %EE %Y||%EE %Y}}' ) },
 	e	=> sub {
-		    $_[2] = $_[0]->day() || $_[0]->holiday();
+		    $_[2] = $_[0]->day_number() || $_[0]->holiday_number();
 		    goto &_fmt_number__2;
 		},
 	F	=> sub { __format( $_[0], '%Y-%{{%m-%d||%Ee}}' ) },
@@ -227,7 +227,7 @@ sub _fmt_get_md {
 		    defined $_[1]{wid}
 			or $_[1]{wid} = 3;
 		    $_[2] = __date_to_day_of_year(
-			$_[0]->year(), _fmt_get_md( $_[0] ) );
+			$_[0]->year_number(), _fmt_get_md( $_[0] ) );
 		    goto &_fmt_number;
 		},
 	k	=> sub { $_[2] = $_[0]->hour(); goto &_fmt_number__2; },
@@ -236,7 +236,7 @@ sub _fmt_get_md {
 		    goto &_fmt_number__2;
 		},
 	M	=> sub { $_[2] = $_[0]->minute(); goto &_fmt_number_02; },
-	m	=> sub { $_[2] = $_[0]->month(); goto &_fmt_number_02; },
+	m	=> sub { $_[2] = $_[0]->month_number(); goto &_fmt_number_02; },
 	N	=> sub {
 		    defined $_[1]{wid}
 			or $_[1]{wid} = 9;
@@ -244,8 +244,8 @@ sub _fmt_get_md {
 		    goto &_fmt_number;
 		},
 	n	=> sub { "\n" },
-	P	=> sub { ( $_[0]->hour() || 0 ) > 11 ? 'pm' : 'am' },
-	p	=> sub { ( $_[0]->hour() || 0 ) > 11 ? 'PM' : 'AM' },
+	P	=> sub { lc __am_or_pm( $_[0]->hour() ) },
+	p	=> sub { uc __am_or_pm( $_[0]->hour() ) },
 	R	=> sub { __format( $_[0], '%H:%M' ) },
 	r	=> sub { __format( $_[0], '%I:%M:%S %p' ) },
 	S	=> sub { $_[2] = $_[0]->second(); goto &_fmt_number_02; },
@@ -256,14 +256,17 @@ sub _fmt_get_md {
 		    $_[2] = __week_of_year( _fmt_get_md( $_[0] ) );
 		    goto &_fmt_number_02;
 		},
-	u	=> sub { $_[0]->day_of_week() },
+	u	=> sub { $_[0]->weekday_number() },
 #	V	Same as U by definition of Shire calendar
 	v	=> sub { __format( $_[0], '%{{%e-%b-%Y||%Ee-%Y}}' ) },
 #	W	Same as U by definition of Shire calendar
 #	X	Same as r, I think
 	x	=> sub { __format( $_[0], '%{{%e %b %Y||%Ee %Y}}' ) }, 
-	Y	=> sub { $_[0]->year() },
-	y	=> sub { $_[2] = $_[0]->year() % 100; goto &_fmt_number_02 },
+	Y	=> sub { $_[0]->year_number() },
+	y	=> sub {
+		    $_[2] = $_[0]->year_number() % 100;
+		    goto &_fmt_number_02;
+		},
 	Z	=> sub { $_[0]->time_zone_short_name() },
 	z	=> sub { _fmt_offset( $_[0]->offset() ) },
 	'%'	=> sub { '%' },
@@ -287,7 +290,7 @@ sub _fmt_get_md {
     );
 
     my %case_change = map { $_ => sub { uc $_[0] } }
-	qw{ A a B b EA Ea EE Ee h };
+	qw{ A a B b EE Ee h };
     $case_change{p} = $case_change{Z} = sub { lc $_[0] };
 
     # Note that if I should choose to implement field widths, the width,
@@ -373,13 +376,14 @@ sub _fmt_offset {
 
 sub _fmt_on_date {
     my ( $date, $ctx ) = @_;
-    my @md = _fmt_get_md( $date );
     my $pfx = "\n" x $ctx->{prefix_new_line_unless_empty};
     $ctx->{prefix_new_line_unless_empty} = 0;
-    defined( my $on_date = delete $ctx->{on_date_accented} ?
-	__on_date_accented( @md ) :
-	__on_date( @md )
-    ) or return undef;	## no critic (ProhibitExplicitReturnUndef)
+    my $month = $date->month_number();
+    my $day = $date->day_number() || $date->holiday_number();
+    defined( my $on_date = $date->accented() ?
+	__on_date_accented( $month, $day ) :
+	__on_date( $month, $day ) )
+	or return undef;	## no critic (ProhibitExplicitReturnUndef)
     return "$pfx$on_date";
 }
 
@@ -389,25 +393,15 @@ sub _fmt_on_date {
 	'1 Yule',
     );
 
+    my $validate_name = _make_validator( qw{ UInt|Undef } );
+
     sub __holiday_name {
-	my ( $holiday ) = @_;
+	my ( $holiday ) = $validate_name->( @_ );
+	defined $holiday
+	    or return [ @name ];
 	return $name[ $holiday || 0 ];
     }
 
-    my $lookup = _make_lookup_hash( @name, 'myd', 'olithe' );
-    my @map = ( 0 .. 6, 3, 4 );
-    foreach ( values %{ $lookup } ) {
-	$_ = $map[$_];
-    }
-    $lookup->{m} = 3;
-    $lookup->{o} = 4;
-
-    sub __holiday_name_to_number {
-	my ( $holiday ) = _normalize_for_lookup( @_ );
-	$holiday =~ m/ \A [0-9]+ \z /smx
-	    and return $holiday;
-	return $lookup->{$holiday} || 0;
-    }
 }
 
 {
@@ -415,15 +409,44 @@ sub _fmt_on_date {
 	'2Yu', '1Li', 'Myd', 'Oli', '2Li', '1Yu',
     );
 
-    sub __holiday_short {
-	my ( $holiday ) = @_;
+    my $validate = _make_validator( qw{ UInt|Undef } );
+
+    sub __holiday_abbr {
+	my ( $holiday ) = $validate->( @_ );
+	defined $holiday
+	    or return [ @name ];
 	return $name[ $holiday || 0 ];
     }
 }
 
-sub __is_leap_year {
-    my ( $year ) = @_;
-    return $year % 4 ? 0 : $year % 100 ? 1 : $year % 400 ? 0 : 1;
+{
+    # This code needs to come after both __holiday_name() and
+    # __holiday_abbr(), because it calls them both and needs the name
+    # arrays to be set up.
+    my $lookup = _make_lookup_hash(
+	__holiday_name(),
+	__holiday_abbr(),
+    );
+
+    my $validate = _make_validator( qw{ Scalar } );
+
+    sub __holiday_name_to_number {
+	my ( $holiday ) = _normalize_for_lookup(
+	    $validate->( @_ ) );
+
+	$holiday =~ m/ \A [0-9]+ \z /smx
+	    and return $holiday;
+	return $lookup->{$holiday} || 0;
+    }
+}
+
+{
+    my $validate = _make_validator( qw{ UInt } );
+
+    sub __is_leap_year {
+	my ( $year ) = $validate->( @_ );
+	return $year % 4 ? 0 : $year % 100 ? 1 : $year % 400 ? 0 : 1;
+    }
 }
 
 {
@@ -433,36 +456,49 @@ sub __is_leap_year {
 	'Blotmath', 'Foreyule',
     );
 
+    my $validate = _make_validator( qw{ UInt|Undef } );
+
     sub __month_name {
-	my ( $month ) = @_;
-	return $name[ $month || 0 ];
+	my ( $month ) = $validate->( @_ );
+	defined $month
+	    or return [ @name ];
+	return $name[ $month ];
     }
 
-    my $lookup = _make_lookup_hash( @name, qw{ ayule flithe alithe fyule
-	} );
-    my @map = ( 0 .. 12, 1, 6, 7, 12 );
-    foreach ( values %{ $lookup } ) {
-	$_ = $map[$_];
-    }
-
-    sub __month_name_to_number {
-	my ( $month ) = _normalize_for_lookup( @_ );
-	$month =~ m/ \A [0-9]+ \z /smx
-	    and return $month;
-	return $lookup->{$month} || 0;
-    }
 }
 
 {
     my @name = ( '', 'Ayu', 'Sol', 'Ret', 'Ast', 'Thr', 'Fli', 'Ali',
 	'Wed', 'Hal', 'Win', 'Blo', 'Fyu' );
 
-    sub __month_short {
-	my ( $month ) = @_;
+    my $validate = _make_validator( qw{ UInt|Undef } );
+
+    sub __month_abbr {
+	my ( $month ) = $validate->( @_ );
+	defined $month
+	    or return [ @name ];
 	return $name[ $month || 0 ];
     }
 }
 
+{
+    my $lookup = _make_lookup_hash(
+	__month_name(),
+	__month_abbr(),
+    );
+
+    my $validate = _make_validator( qw{ Scalar } );
+
+    sub __month_name_to_number {
+	my ( $month ) = _normalize_for_lookup(
+	    $validate->( @_ ) );
+
+	$month =~ m/ \A [0-9]+ \z /smx
+	    and return $month;
+	return $lookup->{$month} || 0;
+    }
+}
+
 {
     my @on_date;
 
@@ -607,9 +643,10 @@ sub __is_leap_year {
 
     $on_date[12][25] = "The Company of the Ring leaves Rivendell at dusk, 1418.\n";
 
+    my $validate = _make_validator( qw{ UInt UInt|Undef } );
 
     sub __on_date {
-	my ( $month, $day ) = @_;
+	my ( $month, $day ) = $validate->( @_ );
 	defined $day
 	    or ( $month, $day ) = ( 0, $month );
 	return $on_date[$month][$day];
@@ -618,7 +655,7 @@ sub __is_leap_year {
     my @on_date_accented;
 
     sub __on_date_accented {
-	my ( $month, $day ) = @_;
+	my ( $month, $day ) = $validate->( @_ );
 	defined $day
 	    or ( $month, $day ) = ( 0, $month );
 
@@ -627,15 +664,16 @@ sub __is_leap_year {
 	    # This would be much easier with 'use utf8;', but
 	    # unfortunately this was broken under Perl 5.6.
 	    my $E_acute	= "\N{LATIN CAPITAL LETTER E WITH ACUTE}";
-	    my $e_acute	= "\N{LATIN CAPITAL LETTER E WITH ACUTE}";
-	    my $o_acute	= "\N{LATIN CAPITAL LETTER O WITH ACUTE}";
-	    my $u_acute	= "\N{LATIN CAPITAL LETTER U WITH ACUTE}";
+	    my $e_acute	= "\N{LATIN SMALL LETTER E WITH ACUTE}";
+	    my $o_acute	= "\N{LATIN SMALL LETTER O WITH ACUTE}";
+	    my $u_acute	= "\N{LATIN SMALL LETTER U WITH ACUTE}";
 	    my $u_circ	= "\N{LATIN SMALL LETTER U WITH CIRCUMFLEX}";
 
 	    foreach my $month ( @on_date ) {
 		push @on_date_accented, [];
 		foreach my $day ( @{ $month } ) {
 		    if ( $day ) {
+			$day =~ s/ \b Anorien \b /An${o_acute}rien/smxgo;
 			$day =~ s/ \b Annun \b /Ann${u_circ}n/smxgo;
 			$day =~ s/ \b Barad-dur \b /Barad-d${u_circ}r/smxgo;
 			$day =~ s/ \b Dunedain \b /D${u_acute}nedain/smxgo;
@@ -659,47 +697,82 @@ sub __is_leap_year {
 {
     my @holiday_quarter = ( undef, 1, 2, 0, 0, 3, 4 );
 
+    my $validate = _make_validator( qw{ UInt UInt|Undef } );
+
     sub __quarter {
-	my ( $month, $day ) = @_;
+	my ( $month, $day ) = $validate->( @_ );
+	defined $day
+	    or ( $month, $day ) = ( 0, $month );
 	return $month ?
 	    POSIX::floor( ( $month - 1 ) / 3 ) + 1 :
 	    $holiday_quarter[$day];
     }
 }
 
-sub __rata_die_to_year_day {
-    my ( $rata_die ) = @_;
+{
+    my @name = ( '', '1st quarter', '2nd quarter', '3rd quarter',
+	'4th quarter' );
 
-    --$rata_die;	# The algorithm is simpler with zero-based days.
-    my $cycle = POSIX::floor( $rata_die / 146097 );
-    my $day_of_cycle = $rata_die - $cycle * 146097;
-    my $year = POSIX::floor( ( $day_of_cycle -
-	    POSIX::floor( $day_of_cycle / 1460 ) +
-	    POSIX::floor( $day_of_cycle / 36524 ) -
-	    POSIX::floor( $day_of_cycle / 146096 ) ) / 365 ) +
-	    400 * $cycle + 1;
-    # We pay here for the zero-based day by having to add back 2 rather
-    # than 1.
-    my $year_day = $rata_die - __year_day_to_rata_die( $year ) + 2;
-    return ( $year, $year_day );
+    my $validate = _make_validator( qw{ UInt } );
+
+    sub __quarter_name {
+	my ( $quarter ) = $validate->( @_ );
+	return $name[ $quarter ];
+    }
+}
+
+{
+    my @name = ( '', qw{ Q1 Q2 Q3 Q4 } );
+
+    my $validate = _make_validator( qw{ UInt } );
+
+    sub __quarter_abbr {
+	my ( $quarter ) = $validate->( @_ );
+	return $name[ $quarter ];
+    }
+}
+
+{
+    my $validate = _make_validator( qw{ Int } );
+
+    sub __rata_die_to_year_day {
+	my ( $rata_die ) = $validate->( @_ );
+
+	--$rata_die;	# The algorithm is simpler with zero-based days.
+	my $cycle = POSIX::floor( $rata_die / 146097 );
+	my $day_of_cycle = $rata_die - $cycle * 146097;
+	my $year = POSIX::floor( ( $day_of_cycle -
+		POSIX::floor( $day_of_cycle / 1460 ) +
+		POSIX::floor( $day_of_cycle / 36524 ) -
+		POSIX::floor( $day_of_cycle / 146096 ) ) / 365 ) +
+		400 * $cycle + 1;
+	# We pay here for the zero-based day by having to add back 2
+	# rather than 1.
+	my $year_day = $rata_die - __year_day_to_rata_die( $year ) + 2;
+	return ( $year, $year_day );
+    }
 }
 
 {
     my @name = ( '', 'Sterrendei', 'Sunnendei', 'Monendei',
 	'Trewesdei', 'Hevenesdei', 'Meresdei', 'Highdei' );
 
+    my $validate = _make_validator( qw{ UInt } );
+
     sub __trad_weekday_name {
-	my ( $weekday ) = @_;
-	return $name[ $weekday || 0 ];
+	my ( $weekday ) = $validate->( @_ );
+	return $name[ $weekday ];
     }
 }
 
 {
     my @name = ( '', 'Ste', 'Sun', 'Mon', 'Tre', 'Hev', 'Mer', 'Hig' );
 
-    sub __trad_weekday_short {
-	my ( $weekday ) = @_;
-	return $name[ $weekday || 0 ];
+    my $validate = _make_validator( qw{ UInt } );
+
+    sub __trad_weekday_abbr {
+	my ( $weekday ) = $validate->( @_ );
+	return $name[ $weekday ];
     }
 }
 
@@ -707,8 +780,10 @@ sub __rata_die_to_year_day {
     my @holiday = ( undef, 1, 26, 0, 0, 27, 52 );
     my @month_offset = ( undef, ( 0 ) x 6, ( 2 ) x 6 );
 
+    my $validate = _make_validator( qw{ UInt UInt } );
+
     sub __week_of_year {
-	my ( $month, $day ) = @_;
+	my ( $month, $day ) = $validate->( @_ );
 	$month
 	    or return $holiday[$day];
 	return int( (
@@ -721,68 +796,273 @@ sub __rata_die_to_year_day {
     my @name = ( '', 'Sterday', 'Sunday', 'Monday', 'Trewsday',
 	'Hevensday', 'Mersday', 'Highday' );
 
+    my $validate = _make_validator( qw{ UInt } );
+
     sub __weekday_name {
-	my ( $weekday ) = @_;
-	return $name[ $weekday || 0 ];
+	my ( $weekday ) = $validate->( @_ );
+	return $name[ $weekday ];
     }
 }
 
 {
     my @name = ( '', 'Ste', 'Sun', 'Mon', 'Tre', 'Hev', 'Mer', 'Hig' );
 
-    sub __weekday_short {
-	my ( $weekday ) = @_;
-	return $name[ $weekday || 0 ];
+    my $validate = _make_validator( qw{ UInt } );
+
+    sub __weekday_abbr {
+	my ( $weekday ) = $validate->( @_ );
+	return $name[ $weekday ];
     }
 }
 
-sub __year_day_to_rata_die {
-    my ( $year, $day ) = @_;
-    --$year;
-    $day ||= 1;
-    return $year * 365 + POSIX::floor( $year / 4 ) -
-	POSIX::floor( $year / 100 ) + POSIX::floor( $year / 400 ) +
-	$day;
+{
+    my $validate = _make_validator( qw{ Int UInt|Undef } );
+
+    sub __year_day_to_rata_die {
+	my ( $year, $day ) = $validate->( @_ );
+	--$year;
+	$day ||= 1;
+	return $year * 365 + POSIX::floor( $year / 4 ) -
+	    POSIX::floor( $year / 100 ) + POSIX::floor( $year / 400 ) +
+	    $day;
+    }
+}
+
+use constant FORMAT_DATE_ERROR => 'Date must be object or hash';
+use constant DATE_CLASS => join '::', __PACKAGE__, 'Date';
+
+sub _make_date_object {
+    my ( $date ) = @_;
+
+    my $ref = ref $date
+	or Carp::croak( FORMAT_DATE_ERROR );
+
+    if ( HASH_REF eq $ref ) {
+	$date = { %{ $date } };	# Clone
+	foreach my $key ( qw{ year month day holiday } ) {
+	    exists $date->{$key}
+		or next;
+	    # TODO alert here and then strip extra code once we're
+	    # stable on this nomenclature
+	    $date->{ "${key}_number" } = delete $date->{$key};
+	}
+	my %hash = %{ $date };
+	if ( $hash{month_number} ) {
+	    $hash{holiday_number} = 0;
+	    $hash{day} ||= 1;
+	} else {
+	    defined $hash{holiday_number}
+		or $hash{holiday_number} = ( $hash{day} || 0 );
+	    $hash{month_number} = $hash{day} = 0;
+	}
+	$hash{$_} ||= 0 for qw{ hour minute second nanosecond };
+	$date = bless \%hash, DATE_CLASS;
+    }
+
+    {
+	my @missing = __valid_date_class( $date );
+	local $" = ', ';
+	@missing
+	    and Carp::croak(
+	    "A @{[ ref $date ]} is not a valid date object, it lacks: @missing" );
+    }
+
+    return $date;
+}
+
+sub __valid_date_class {
+    my ( $obj ) = @_;
+    my $package = ref $obj || $obj;
+    unless ( ref $obj ) {
+	( my $fn = $package ) =~ s{ :: }{/}smxg;
+	$fn .= '.pm';
+	$INC{$fn}
+	    or require $fn;
+    }
+    my @missing;
+    foreach my $method ( qw{
+	year_number
+	month_number
+	day_number
+	holiday_number
+	hour
+	minute
+	second
+	weekday_number
+	nanosecond
+	epoch
+	offset
+	time_zone_short_name
+	accented
+	traditional
+    } ) {
+	$package->can( $method )
+	    or push @missing, $method;
+    }
+    return @missing;
+}
+
+# The arguments are multiple array references. The hash is set up so
+# that all unique abbreviations of elements 0 return 0, and so on. The
+# respective elements at the same index do not conflict with each other,
+# so that (to take a not-so-random example) if two arrays are passed in,
+# and the respective element 3s are (after normalization) 'midyearsday'
+# and 'myd', and no other entries start with 'm', then key 'm' will
+# exist and return value 3.
+sub _make_lookup_hash {
+    my @sources = @_;
+    my %conflict;
+    my %merged;
+    my $source_count;
+    foreach ( @sources ) {
+	my @source = _normalize_for_lookup( @{ $_ } );
+	my %value;
+	foreach my $inx ( 0 .. $#source ) {
+	    $value{ $source[$inx] } = $inx;
+	}
+	my %hash = Text::Abbrev::abbrev( @source );
+	delete $hash{''};
+	foreach ( values %hash ) {
+	    $_ = $value{$_};
+	}
+	# Would use keys %merged here, but not sure how performant that
+	# is under older Perls.
+	if ( $source_count++ ) {
+	    foreach my $key ( keys %hash ) {
+		if ( $conflict{$key} ) {
+		    # ignore it
+		} elsif ( $merged{$key} ) {
+		    if ( $merged{$key} != $hash{$key} ) {
+			delete $merged{$key};
+			$conflict{$key} = 1;
+		    }
+		} else {
+		    $merged{$key} = $hash{$key};
+		}
+	    }
+	} else {
+	    %merged = %hash;
+	}
+    }
+    return wantarray ? %merged : \%merged;
+}
+
+# I want this module to be light weight, but I also want to limit the
+# arguments so I can add or change them with confidence that I don't
+# break anything. So this is poor man's validation.
+{
+    my %type_def;
+
+    BEGIN {
+	# Type definitions expect the value begin validated to be in $_.
+	# They return false if the value passes the validation, and a
+	# brief description of what was expected (which must be a true
+	# value as far as Perl is concerned) if the value fails
+	# validation. They must not throw exceptions, because an
+	# individual validator may be part of an alternation.
+	#
+	# We need the BEGIN block because we are manufacturing
+	# validators in-line, above, and %type_def needs to be populated
+	# before that happens.
+	%type_def = (
+	    # A hash reference
+	    Hash	=> sub { HASH_REF eq ref $_ ? 0 : 'a HASH reference' },
+	    # An integer, optionally signed
+	    Int	=> sub {
+		( defined $_ && m/ \A [-+]? [0-9]+ \z /smx ) ? 0 :
+		'an integer';
+	    },
+	    # An object (i.e. a blessed reference). I am not using
+	    # Scalar::Util::blessed() here because of the desire to run
+	    # under versions of Perl before this was released to core.
+	    Object	=> sub {
+		local $@ = undef;
+		( ref $_ && eval { $_->can( 'isa' ) } ) ? ## no critic (RequireCheckingReturnValueOfEval)
+		0 : 'an object' },
+	    # A defined scalar (i.e. not a reference)
+	    Scalar	=> sub { ( defined $_ && ! ref $_ ) ? 0 :
+		'a non-reference' },
+	    # An unsigned integer
+	    UInt	=> sub {
+		( defined $_ && m/ \A [0-9]+ \z /smx ) ? 0 :
+		'an unsigned integer';
+	    },
+	    # Undefined. Necessary because all the other types reject an
+	    # undefined value.
+	    Undef	=> sub { defined $_ ? 'undefined' : 0 },
+	);
+    }
+
+    # Take as arguments the type specifications of all arguments of the
+    # subroutine to be validated, and return a reference to code that
+    # checks its arguments against those specs. Type specifications must
+    # appear in the above table, or be an alternation of items in the
+    # above table (i.e. joined by '|', e.g. 'Scalar|Undef').
+    #
+    # There is currently no way to do slurpy arguments.
+    sub _make_validator {
+	my ( @spec ) = @_;
+	foreach my $inx ( 0 .. $#spec ) {
+	    foreach my $type ( split qr{ [|] }smx, $spec[$inx] ) {
+		$type_def{$type}
+		    or Carp::confess(
+		    "Programming error - Argument $inx type '$spec[$inx]' is unknown" );
+	    }
+	}
+	return sub {
+	    my @args = @_;
+	    @args > @spec
+		and Carp::croak( 'Too many arguments' );
+	ARGUMENT_LOOP:
+	    foreach my $inx ( 0 .. $#spec ) {
+		my @fail;
+		local $_ = $args[$inx];
+		foreach my $type ( split qr{ [|] }smx, $spec[$inx] ) {
+		    my $error = $type_def{$type}->()
+			or next ARGUMENT_LOOP;
+		    push @fail, $error;
+		}
+		local $" = ' or ';
+		Carp::croak( "Argument $inx ('$_') must be @fail" );
+	    }
+	    return @args;
+	};
+    }
 }
 
 sub _normalize_for_lookup {
     my @data = @_;
     foreach ( @data ) {
-	( $_ = lc $_ ) =~ s/ [\s[:punct:]]+ //smxg;
+	defined $_
+	    and ( $_ = lc $_ ) =~ s/ [\s[:punct:]]+ //smxg;
     }
     return @data;
 }
 
-sub _make_lookup_hash {
-    my @data = _normalize_for_lookup( @_ );
-    my %index;
-    for ( my $inx = 0; $inx < @data; $inx++ ) {
-	$index{$data[$inx]} = $inx;
-    }
-    my %hash = Text::Abbrev::abbrev( @data );
-    delete $hash{ '' };
-    foreach ( values %hash ) {
-	$_ = $index{$_};
-    }
-    return wantarray ? %hash : \%hash;
-}
-
 # Create methods for the hash wrapper
 
 {
     my %calc = (
-	day_of_week	=> sub {
+	weekday_number	=> sub {
 	    return __day_of_week(
-		$_[0]->month(),
-		$_[0]->day() || $_[0]->holiday(),
+		$_[0]->month_number(),
+		$_[0]->day_number() || $_[0]->holiday_number(),
+	    );
+	},
+	quarter		=> sub {
+	    return __quarter(
+		$_[0]->month_number(),
+		$_[0]->day_number() || $_[0]->holiday_number(),
 	    );
 	},
     );
 
     foreach my $method ( qw{
-	year month day holiday hour minute second nanosecond
-	epoch offset time_zone_short_name day_of_week
-    } ) {
+	year_number month_number day_number holiday_number
+	hour minute second nanosecond epoch
+	offset time_zone_short_name
+	accented traditional
+    }, keys %calc ) {
 	my $fqn = join '::', __PACKAGE__, 'Date', $method;
 	if ( my $code = $calc{$method} ) {
 	    no strict qw{ refs };
@@ -814,12 +1094,16 @@ Date::Tolkien::Shire::Data - Data functionality for Shire calendars.
 
 =head1 DESCRIPTION
 
+UNTIL THE VERSION 0.001 I CONSIDER THIS ALPHA CODE, AND THE INTERFACE
+CAN AND WILL CHANGE WITHOUT NOTICE. CAVEAT CODER.
+
 This Perl module carries common functionality for implementations of the
 Shire calendar as described in Appendix D of J. R. R. Tolkien's novel
 "Lord Of The Rings". What it really contains is anything that was common
 to L<Date::Tolkien::Shire|Date::Tolkien::Shire> and
 L<DateTime::Fiction::JRRTolkien::Shire|DateTime::Fiction::JRRTolkien::Shire>
-and I felt like factoring out.
+and I felt like factoring out. You probably do not want to use this
+module directly, at least not without looking into the other two.
 
 The Shire calendar has 12 months of 30 days each, plus 5 holidays (6 in
 a leap year) that are not part of any month. Two of these holidays
@@ -858,6 +1142,11 @@ that uses this package.
 All of the following are exportable to your name space, but none are
 exported by default.
 
+=head2 __am_or_pm
+
+This subroutine takes as input an hour of the day in the range 0 to 23
+and returns C<'AM'> if the hour is less than 12, or C<'PM'> otherwise.
+
 =head2 __date_to_day_of_year
 
  say __date_to_day_of_year( 1420, 3, 25 );
@@ -885,21 +1174,25 @@ between C<1> and C<365>, or C<366> in a leap year.
 
 =head2 __format
 
- say __format( $date, '%c' );
+ say __format( $date, $pattern );
 
 This method formats a date, in a manner similar to C<strftime()>. The
 C<$date> is either an object that supports the necessary methods, or a
 reference to a hash having the necessary keys (same as the
-methods). The methods (or keys) used are:
+methods). The C<$pattern> is a string similar to the conversion
+specification passed to C<POSIX::strftime()>; see below for a fuller
+description.
 
-  year
-  month
-  day
-  holiday
+The C<$date> methods (or keys) used are:
+
+  year_number
+  month_number
+  day_number
+  holiday_number
   hour
   minute
   second
-  day_of_week
+  weekday_number
   nanosecond
   epoch
   offset
@@ -908,13 +1201,14 @@ methods). The methods (or keys) used are:
 The first seven are heavily used. The last four are used only by
 C<'%N'>, C<'%s'>, C<'%z'>, and C<'%Z'> respectively.
 
-If you pass a hash, the canonical say specification is either a
-non-zero C<month> and C<day> and zero C<holiday>, or vice versa. But if
-you pass a zero C<month> and C<holiday>, the method will assume that you
-were specifying a holiday and adjust the hash accordingly, though this
-adjustment will not be visible outside C<__format()>. Also, your hash
-need not specify C<day_of_week> since it can be computed from C<month>
-and C<day>, or C<holiday>.
+If you pass a hash, the canonical day specification is either a non-zero
+C<month_number> and C<day> and zero C<holiday_number>, or vice versa.
+But if you pass a zero C<month_number> and C<holiday_number>, the method
+will assume that you were specifying a holiday_number and adjust the
+hash accordingly, though this adjustment will not be visible outside
+C<__format()>. Also, your hash need not specify C<weekday_number> since
+it can be computed from C<month_number> and C<day>, or
+C<holiday_number>.
 
 The following conversion specifications (to use C<strftime()>
 terminology) or patterns (to use L<DateTime|DateTime> terminology) are
@@ -973,23 +1267,6 @@ The day of the month as a decimal number, zero-filled (range C<01> to
 C<30>). On holidays it is the holiday number, zero-filled (range C<01>
 to C<06>).
 
-=item %EA
-
-The full traditional weekday name, or C<''> for holidays that are part
-of no week.
-
-=item %Ea
-
-The abbreviated traditional weekday name, or C<''> for holidays that are
-part of no week.
-
-=item %ED
-
-The L<__on_date_accented()|/__on_date_accented> text for the given date.
-
-You can get a leading C<"\n"> if there was an actual event using
-C<'%En%ED'>.
-
 =item %Ed
 
 The L<__on_date()|/__on_date> text for the given date.
@@ -1008,9 +1285,8 @@ The abbreviated holiday name, or C<''> for non-holidays.
 
 =item %En
 
-Inserts nothing, but causes the next C<%Ed> or C<%ED> (and B<only> the
-next one) to have a C<"\n"> prefixed if there was an actual event on the
-date.
+Inserts nothing, but causes the next C<%Ed> (and B<only> the next one)
+to have a C<"\n"> prefixed if there was an actual event on the date.
 
 =item %Ex
 
@@ -1251,9 +1527,8 @@ If applied to C<'%p'> or C<'%Z'>, this flag converts the output to lower
 case. It also overrides the C<'^'> flag if both are specified,
 regardless of order.
 
-If applied to C<'%A'>, C<'%a'>, C<'%B'>, C<'%b'> C<'%EA'>, C<'%Ea'>
-C<'%EE'>, C<'%Ee'>, or C<'%h'>, this flag converts the output to lower
-case.
+If applied to C<'%A'>, C<'%a'>, C<'%B'>, C<'%b'>, C<'%EE'>, C<'%Ee'>, or
+C<'%h'>, this flag converts the output to upper case.
 
 If applied to anything else, this flag has no effect.
 
@@ -1282,16 +1557,15 @@ number of the holiday. Unique abbreviations of names or short names
 digits are returned unmodified. Anything unrecognized causes C<0> to be
 returned.
 
-=head2 __holiday_short
+=head2 __holiday_abbr
 
- say __holiday_short( 3 );
+ say __holiday_abbr( 3 );
 
 Given a holiday number C<(1-6)>, this subroutine returns that holiday's
 three-letter abbreviation. If the holiday number is C<0> (i.e. the day
 is not a holiday), an empty string is returned. Otherwise, C<undef> is
 returned.
 
-
 =head2 __is_leap_year
 
  say __is_leap_year( 1420 );  # 1
@@ -1318,9 +1592,9 @@ number of the month. Unique abbreviations of names or short names
 digits are returned unmodified. Anything unrecognized causes C<0> to be
 returned.
 
-=head2 __month_short
+=head2 __month_abbr
 
- say __month_short( 3 );
+ say __month_abbr( 3 );
 
 Given a month number C<(1-12)>, this subroutine returns that month's
 three-letter abbreviation. If the month number is C<0> (i.e. a holiday),
@@ -1367,6 +1641,17 @@ There is nothing I know of about hobbits using calendar quarters in
 anything Tolkien wrote. But if they did use them I suspect they would be
 rationalized this way.
 
+=head2 __quarter_name
+
+Given the quarter number, return the name of the quarter. If the quarter
+number is C<0> (i.e. Midyear's day or Overlithe), C<''> is returned.
+
+=head2 __quarter_abbr
+
+Given the quarter number, return the abbreviated name of the quarter. If
+the quarter number is C<0> (i.e. Midyear's day or Overlithe), C<''> is
+returned.
+
 =head2 __rata_die_to_year_day
 
  my ( $year, $day ) = __rata_die_to_year_day( $rata_die );
@@ -1397,15 +1682,33 @@ weekday given its number (1-7). If the weekday number is C<0> (i.e.
 Midyear's day or the Overlithe) the empty string is returned. Otherwise,
 C<undef> is returned.
 
-=head2 __trad_weekday_short
+=head2 __trad_weekday_abbr
 
- say 'Day 1 is ', __trad_weekday_short( 1 );
+ say 'Day 1 is ', __trad_weekday_abbr( 1 );
 
 This subroutine computes the three-letter abbreviation of a traditional
 (i.e. old-style) weekday given its number (1-7). If the weekday number
 is C<0> (i.e.  Midyear's day or the Overlithe) the empty string is
 returned. Otherwise, C<undef> is returned.
 
+=head2 __valid_date_class
+
+This subroutine takes as its argument either an object or a class name.
+If a class name, it is loaded if possible, and if it has not already
+been loaded. Then the class is checked to see if it has all the methods
+required of a date object passed to L<__format()|/__format>.
+
+In list context it returns the names of any missing methods. In scalar
+context it returns the number of missing methods. So a return of an
+empty list (or zero in scalar context) means the class can probably be
+passed as a date to L<__format()|/__format()>. The weasel word
+'probably' is because the check is on method name only; we have no idea
+whether the semantics of the methods found are what we would like them
+to be.
+
+This subroutine is exposed for troubleshooting purposes, though it is
+hoped the exception thrown if this check fails will be sufficiently
+explicit. See also F<tools/valid-date-class>.
 
 =head2 __weekday_name
 
@@ -1415,9 +1718,9 @@ This subroutine computes the name of a weekday given its number (1-7).
 If the weekday number is C<0> (i.e. Midyear's day or the Overlithe) the
 empty string is returned. Otherwise, C<undef> is returned.
 
-=head2 __weekday_short
+=head2 __weekday_abbr
 
- say 'Day 1 is ', __weekday_short( 1 );
+ say 'Day 1 is ', __weekday_abbr( 1 );
 
 This subroutine computes the three-letter abbreviation of a weekday
 given its number (1-7). If the weekday number is C<0> (i.e. Midyear's
@@ -1435,6 +1738,9 @@ returned.
 
 =head2 __year_day_to_rata_die
 
+ my $rata_die = __year_day_to_rata_die(
+     $year, $day_of_year );
+
 Given the year and day of the year, this subroutine returns the Rata Die
 day of the given year and day. The day of the year defaults to C<1>.
 
